package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouCrowdPack;
import cn.com.ctop.kuaishou.modules.ai.mapper.KuaishouCrowdPackMapper;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouCrowdPackService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouPopulationService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
public class CrowdListQueryJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private KuaishouCrowdPackMapper crowdPackMapper;
    @Autowired
    private IKuaishouPopulationService iKuaishouPopulationService;
    @Autowired
    private IKuaishouCrowdPackService crowdPackService;

    static ExecutorService executorService = Executors.newFixedThreadPool(5);

    @XxlJob("crowdListQueryJob")
    public void execute() {
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i <= 7; i++) {
            if (i != 2) {
                list.add(i);
            }
        }
        list.forEach(status -> executorService.submit(() -> {
            List<KuaishouCrowdPack> crowdList = crowdPackMapper.queryListByStatus();
            for (KuaishouCrowdPack crowdPack : crowdList) {
                CtopOauthToken token = tokenService.getTokenByAccountId(crowdPack.getAccountId());
                JSONObject jsonObject = iKuaishouPopulationService.getPopulationList(crowdPack.getAccountId(), token.getAccessToken(), status);
                if (jsonObject.getInteger("code") == 0) {
                    JSONObject data = jsonObject.getJSONObject("data");
                    if (!Check.isNull(data)) {
                        JSONArray details = data.getJSONArray("details");
                        if (details.size() > 0) {
                            for (int j = 0; j < details.size(); j++) {
                                JSONObject obj = details.getJSONObject(j);
                                if (Check.isNull(crowdPackMapper.queryByAccountId(obj.getLong("account_id"), obj.getLong("orientation_id")))) {
                                    KuaishouCrowdPack oldPack = crowdPackMapper.queryByAccountId(null, obj.getLong("orientation_id"));
                                    KuaishouCrowdPack crowdPackEntity = JSONObject.toJavaObject(obj, KuaishouCrowdPack.class);
                                    if (!Check.isNull(oldPack)) {
                                        crowdPackEntity.setStatDate(oldPack.getStatDate());
                                        crowdPackEntity.setUrl(oldPack.getUrl());
                                        crowdPackEntity.setSignature(oldPack.getSignature());
                                    }
                                    crowdPackEntity.setMessage("PUSH OK");
                                    crowdPackEntity.setDataStatus(1);
                                    crowdPackService.save(crowdPackEntity);
                                } else {
                                    crowdPackMapper.update(obj);
                                }
                            }
                        }
                    }
                }
            }
        }));
        XxlJobHelper.log("快手人群报表更新成功");
    }
}
