package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouCampaignLevelOperationRecord;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouCampaignLevelOperationRecordService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 计划创建记录
 * @author jeecg-boot
 * @date   2021-04-12
 * @version V1.0
 */
@Slf4j
@Api(tags="计划创建记录")
@RestController
@RequestMapping("/ctop/aiKuaishouCampaignLevelOperationRecord")
public class AiKuaishouCampaignLevelOperationRecordController {
	@Autowired
	private IAiKuaishouCampaignLevelOperationRecordService aiKuaishouCampaignLevelOperationRecordService;

	/**
	  * 分页列表查询
	 * @param aiKuaishouCampaignLevelOperationRecord
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<AiKuaishouCampaignLevelOperationRecord>> queryPageList(AiKuaishouCampaignLevelOperationRecord aiKuaishouCampaignLevelOperationRecord,
																			   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
																			   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
																			   HttpServletRequest req) {
		Result<IPage<AiKuaishouCampaignLevelOperationRecord>> result = new Result<>();
		QueryWrapper<AiKuaishouCampaignLevelOperationRecord> queryWrapper = QueryGenerator.initQueryWrapper(aiKuaishouCampaignLevelOperationRecord, req.getParameterMap());
		Page<AiKuaishouCampaignLevelOperationRecord> page = new Page<AiKuaishouCampaignLevelOperationRecord>(pageNo, pageSize);
		IPage<AiKuaishouCampaignLevelOperationRecord> pageList = aiKuaishouCampaignLevelOperationRecordService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param aiKuaishouCampaignLevelOperationRecord
	 * @return
	 */
	@AutoLog(value = "计划创建记录-添加")
	@ApiOperation(value="计划创建记录-添加", notes="计划创建记录-添加")
	@PostMapping(value = "/add")
	public Result<AiKuaishouCampaignLevelOperationRecord> add(@RequestBody AiKuaishouCampaignLevelOperationRecord aiKuaishouCampaignLevelOperationRecord) {
		Result<AiKuaishouCampaignLevelOperationRecord> result = new Result<>();
		try {
			aiKuaishouCampaignLevelOperationRecordService.save(aiKuaishouCampaignLevelOperationRecord);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param aiKuaishouCampaignLevelOperationRecord
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<AiKuaishouCampaignLevelOperationRecord> edit(@RequestBody AiKuaishouCampaignLevelOperationRecord aiKuaishouCampaignLevelOperationRecord) {
		Result<AiKuaishouCampaignLevelOperationRecord> result = new Result<AiKuaishouCampaignLevelOperationRecord>();
		AiKuaishouCampaignLevelOperationRecord aiKuaishouCampaignLevelOperationRecordEntity = aiKuaishouCampaignLevelOperationRecordService.getById(aiKuaishouCampaignLevelOperationRecord.getCampaignUuid());
		if(aiKuaishouCampaignLevelOperationRecordEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = aiKuaishouCampaignLevelOperationRecordService.updateById(aiKuaishouCampaignLevelOperationRecord);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			aiKuaishouCampaignLevelOperationRecordService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "计划创建记录-批量删除")
	@ApiOperation(value="计划创建记录-批量删除", notes="计划创建记录-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<AiKuaishouCampaignLevelOperationRecord> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<AiKuaishouCampaignLevelOperationRecord> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.aiKuaishouCampaignLevelOperationRecordService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<AiKuaishouCampaignLevelOperationRecord> queryById(@RequestParam(name="id",required=true) String id) {
		Result<AiKuaishouCampaignLevelOperationRecord> result = new Result<>();
		AiKuaishouCampaignLevelOperationRecord aiKuaishouCampaignLevelOperationRecord = aiKuaishouCampaignLevelOperationRecordService.getById(id);
		if(aiKuaishouCampaignLevelOperationRecord==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(aiKuaishouCampaignLevelOperationRecord);
			result.setSuccess(true);
		}
		return result;
	}
}
