package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouUnitLevelOperationRecord;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouUnitLevelOperationRecordService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 组创建记录
 * @author jeecg-boot
 * @date   2021-04-12
 * @version V1.0
 */
@Slf4j
@Api(tags="组创建记录")
@RestController
@RequestMapping("/ctop/aiKuaishouUnitLevelOperationRecord")
public class AiKuaishouUnitLevelOperationRecordController {
	@Autowired
	private IAiKuaishouUnitLevelOperationRecordService aiKuaishouUnitLevelOperationRecordService;

	/**
	  * 分页列表查询
	 * @param aiKuaishouUnitLevelOperationRecord
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<AiKuaishouUnitLevelOperationRecord>> queryPageList(AiKuaishouUnitLevelOperationRecord aiKuaishouUnitLevelOperationRecord,
																		   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
																		   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
																		   HttpServletRequest req) {
		Result<IPage<AiKuaishouUnitLevelOperationRecord>> result = new Result<>();
		QueryWrapper<AiKuaishouUnitLevelOperationRecord> queryWrapper = QueryGenerator.initQueryWrapper(aiKuaishouUnitLevelOperationRecord, req.getParameterMap());
		Page<AiKuaishouUnitLevelOperationRecord> page = new Page<AiKuaishouUnitLevelOperationRecord>(pageNo, pageSize);
		IPage<AiKuaishouUnitLevelOperationRecord> pageList = aiKuaishouUnitLevelOperationRecordService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param aiKuaishouUnitLevelOperationRecord
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<AiKuaishouUnitLevelOperationRecord> add(@RequestBody AiKuaishouUnitLevelOperationRecord aiKuaishouUnitLevelOperationRecord) {
		Result<AiKuaishouUnitLevelOperationRecord> result = new Result<>();
		try {
			aiKuaishouUnitLevelOperationRecordService.save(aiKuaishouUnitLevelOperationRecord);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param aiKuaishouUnitLevelOperationRecord
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<AiKuaishouUnitLevelOperationRecord> edit(@RequestBody AiKuaishouUnitLevelOperationRecord aiKuaishouUnitLevelOperationRecord) {
		Result<AiKuaishouUnitLevelOperationRecord> result = new Result<AiKuaishouUnitLevelOperationRecord>();
		AiKuaishouUnitLevelOperationRecord aiKuaishouUnitLevelOperationRecordEntity = aiKuaishouUnitLevelOperationRecordService.getById(aiKuaishouUnitLevelOperationRecord.getGroupUuid());
		if(aiKuaishouUnitLevelOperationRecordEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = aiKuaishouUnitLevelOperationRecordService.updateById(aiKuaishouUnitLevelOperationRecord);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			aiKuaishouUnitLevelOperationRecordService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<AiKuaishouUnitLevelOperationRecord> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<AiKuaishouUnitLevelOperationRecord> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.aiKuaishouUnitLevelOperationRecordService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<AiKuaishouUnitLevelOperationRecord> queryById(@RequestParam(name="id",required=true) String id) {
		Result<AiKuaishouUnitLevelOperationRecord> result = new Result<>();
		AiKuaishouUnitLevelOperationRecord aiKuaishouUnitLevelOperationRecord = aiKuaishouUnitLevelOperationRecordService.getById(id);
		if(aiKuaishouUnitLevelOperationRecord==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(aiKuaishouUnitLevelOperationRecord);
			result.setSuccess(true);
		}
		return result;
	}
}
