package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouCrowdPack;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouCrowdPackService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 人群报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-03-26
 */
@Slf4j
@Api(tags = "人群报表")
@RestController
@RequestMapping("/kuaishouCrowdPack")
public class KuaishouCrowdPackController {
    @Autowired
    private IKuaishouCrowdPackService kuaishouCrowdPackService;

    /**
     * 分页列表查询
     *
     * @param kuaishouCrowdPack
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "人群报表-分页列表查询")
    @ApiOperation(value = "人群报表-分页列表查询", notes = "人群报表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouCrowdPack>> list(KuaishouCrowdPack kuaishouCrowdPack,
                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                 HttpServletRequest req) {
        Result<IPage<KuaishouCrowdPack>> result = new Result<>();
        QueryWrapper<KuaishouCrowdPack> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouCrowdPack, req.getParameterMap());
        Page<KuaishouCrowdPack> page = new Page<KuaishouCrowdPack>(pageNo, pageSize);
        IPage<KuaishouCrowdPack> pageList = kuaishouCrowdPackService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 人群包列表
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/queryPageList")
    public Result<Object> queryPageList(KuaishouCrowdPack kuaishouCrowdPack,
                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            return kuaishouCrowdPackService.queryPageList(kuaishouCrowdPack, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 相关账户查询
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/queryRel")
    public Result<Object> queryRel(String type, String signature,
                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(type) || Check.isNull(signature)) {
                return Result.error("缺失参数");
            }
            return kuaishouCrowdPackService.queryRel(type, signature, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 查询原因
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/queryFailReason")
    public Result<Object> queryFailReason(String signature,
                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            return kuaishouCrowdPackService.queryFailReason(signature, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 上传前的人群包校验
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @GetMapping(value = "/checkCrowdPackage")
    public Result<Object> checkCrowdPackage(String signature) {
        try {
            return kuaishouCrowdPackService.checkCrowdPackage(signature);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 上传人群包
     *
     * @param kuaishouCrowdPack
     * @return
     */

    @PostMapping(value = "/uploadCrowdPack")
    public Result<Object> uploadCrowdPack(@RequestBody JSONObject request) {
        Integer type = request.getInteger("type");
        String orientationName = request.getString("orientationName");
        Long accountId = request.getLong("accountId");
        String url = request.getString("url");
        try {
            if (Check.isNull(type) || Check.isNull(orientationName) || Check.isNull(url) || Check.isNull(accountId)) {
                return Result.error("缺少参数");
            }
            return kuaishouCrowdPackService.uploadCrowdPack(type, orientationName, accountId, url);
        } catch (Exception e) {
            log.error("上传异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("上传失败");
    }

    /**
     * 账户推送
     *
     * @param kuaishouCrowdPack
     * @return
     */
    @AutoLog(value = "账户推送")
    @ApiOperation(value = "账户推送", notes = "账户推送")
    @PostMapping(value = "/accountPush")
    public Result<Object> accountPush(@RequestBody JSONObject request) {
        try {
            JSONArray accountIds = request.getJSONArray("accountIds");
            String signature = request.getString("signature");
            if (Check.isNull(accountIds) || Check.isNull(signature)) {
                return Result.error("缺少参数");
            }
            return kuaishouCrowdPackService.accountPush(request);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    @RequestMapping(value = "/crowdListQuery")
    public Result<Object> crowdListQuery() {
        try {
            kuaishouCrowdPackService.crowdListQuery();
            return Result.ok("success");
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 添加
     *
     * @param kuaishouCrowdPack
     * @return
     */
    @AutoLog(value = "人群报表-添加")
    @ApiOperation(value = "人群报表-添加", notes = "人群报表-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouCrowdPack> add(@RequestBody KuaishouCrowdPack kuaishouCrowdPack) {
        Result<KuaishouCrowdPack> result = new Result<>();
        try {
            kuaishouCrowdPackService.save(kuaishouCrowdPack);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouCrowdPack
     * @return
     */
    @AutoLog(value = "人群报表-编辑")
    @ApiOperation(value = "人群报表-编辑", notes = "人群报表-编辑")
    @PostMapping(value = "/edit")
    public Result<KuaishouCrowdPack> edit(@RequestBody KuaishouCrowdPack kuaishouCrowdPack) {
        Result<KuaishouCrowdPack> result = new Result<KuaishouCrowdPack>();
        KuaishouCrowdPack kuaishouCrowdPackEntity = kuaishouCrowdPackService.getById(kuaishouCrowdPack.getId());
        if (kuaishouCrowdPackEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouCrowdPackService.updateById(kuaishouCrowdPack);
            if (ok) {
                result.success("修改成功!");
            }
        }
        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "人群报表-通过id删除")
    @ApiOperation(value = "人群报表-通过id删除", notes = "人群报表-通过id删除")
    @GetMapping(value = "/delete")
    public Result<Object> delete(KuaishouCrowdPack kuaishouCrowdPack) {
        KuaishouCrowdPack kuaishouCrowdPackEntity = kuaishouCrowdPackService.getById(kuaishouCrowdPack.getId());
        if (kuaishouCrowdPackEntity == null) {
            return Result.error("未找到对应实体");
        } else {
            kuaishouCrowdPackEntity.setDataStatus(0);
            return kuaishouCrowdPackService.updateDataStatus(kuaishouCrowdPackEntity);
        }
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "人群报表-批量删除")
    @ApiOperation(value = "人群报表-批量删除", notes = "人群报表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouCrowdPack> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouCrowdPack> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouCrowdPackService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "人群报表-通过id查询")
    @ApiOperation(value = "人群报表-通过id查询", notes = "人群报表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouCrowdPack> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouCrowdPack> result = new Result<>();
        KuaishouCrowdPack kuaishouCrowdPack = kuaishouCrowdPackService.getById(id);
        if (kuaishouCrowdPack == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouCrowdPack);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouCrowdPack> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouCrowdPack kuaishouCrowdPack = JSON.parseObject(deString, KuaishouCrowdPack.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouCrowdPack, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouCrowdPack> pageList = kuaishouCrowdPackService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "人群报表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouCrowdPack.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("人群报表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouCrowdPack> listKuaishouCrowdPacks = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouCrowdPack.class, params);
                kuaishouCrowdPackService.saveBatch(listKuaishouCrowdPacks);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouCrowdPacks.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
